@@ -20,42 +20,101 @@ import pomsets.parameter as ParameterModule
 
 
 
+# here's the issue that we have with a task's execution status
+# the status really should be an attribute of the task
+# however, we need to notify of when that status changes
+# and right now we are using mext/Reaction
+# and thus we are putting that value into a pypatterns.relational.Table
+# which then notifies a watcher that the execution status has changed
+# but by doing so, that means that for the root task,
+# where there is no parent task, that there is nothing keeping track
+# of its execution status
+
 
 class Task(DefinitionModule.ParameterBindingsHolder, TaskModule.Task):
 
     ATTRIBUTES = DefinitionModule.ParameterBindingsHolder.ATTRIBUTES + [
-        'definition', 
-        'threadPool', # we may not need this, as the automaton holds the threadpool
-        'parentTask',
-        'workRequest',
+        'definition', # the task definition
+        'threadPool', # the threadpool in which the task will run
+        'parentTask', # the parent task of this task (None if root)
+        'workRequest', # all the info about running the task
         'automaton',
+        'executeStepResults', # the results of each execution step of the task
+        'currentExecuteStepIndex', # the index of the current execution step
+        'executeStatus', # the execution status
     ]
+
+    EXECUTE_STATUS_UNINITIALIZED = 'uninitialized'
+    EXECUTE_STATUS_RUNNING = 'running'
+    EXECUTE_STATUS_COMPLETED = 'completed'
+    EXECUTE_STATUS_ERRORED = 'errored'
+    EXECUTE_STATUS_KILLED = 'killed'
     
     def __init__(self):
 
         DefinitionModule.ParameterBindingsHolder.__init__(self)
         TaskModule.Task.__init__(self)
         
+        self.initializeExecuteStageIndex()
+        self.executeStepResults({})
+
+        self.executeStatus(Task.EXECUTE_STATUS_UNINITIALIZED)
         pass
 
-    
+
     def hasParentTask(self):
         return self.parentTask() is not None
+
+
+    def notifyParentOnExecutionStart(self):
+
+        # TODO:
+        # should separate out the notification to the parent
+        # from the setting of the status
+
+        self.executeStatus(Task.EXECUTE_STATUS_RUNNING)
+        if self.hasParentTask():
+            self.parentTask().childTaskIsRunning(self)
+        return
+
     
     def notifyParentOfCompletion(self):
+
+        # TODO:
+        # should separate out the notification to the parent
+        # from the setting of the status
+
+        self.executeStatus(Task.EXECUTE_STATUS_COMPLETED)
         if self.hasParentTask():
             self.parentTask().childTaskHasCompleted(self)
         return
 
+
     def notifyParentOfError(self, errorInfo):
+
+        # TODO:
+        # should separate out the notification to the parent
+        # from the setting of the status
+
+        # if it wasn't killed
+        # then we'll use the generic error status
+        if not self.executeStatus() == Task.EXECUTE_STATUS_KILLED:
+            self.executeStatus(Task.EXECUTE_STATUS_ERRORED)
+
         if self.hasParentTask():
             self.parentTask().childTaskHasErrored(self, errorInfo)
         return
 
+
+    def initializeExecuteStageIndex(self):
+        self.currentExecuteStepIndex(0)
+        return
+
+
     def getParameterBinding(self, key):
         if not self.hasParameterBinding(key):
             if not self.definition().hasParameterBinding(key):
-                raise KeyError('%s not in %s\'s parameter bindings' % (key, self))
+                raise KeyError('%s not in %s\'s parameter bindings' % (key, self.definition().name()))
             return self.definition().getParameterBinding(key)
         return DefinitionModule.ParameterBindingsHolder.getParameterBinding(self, key)
 
@@ -96,9 +155,10 @@ class Task(DefinitionModule.ParameterBindingsHolder, TaskModule.Task):
         """
         these functions take the perspective of the child as the active agent
 
-        data is pulled by the child from the blackboard parameters
-        and pushed by the child back to blackboard parameters
+        data is pulled by the child from the blackboard parameters of the parent
+        and pushed by the child back to blackboard parameters of the parent
         """
+
         if self.hasParentTask():
             self.parentTask().pullDataForParametersOfChild(self)
         return
@@ -150,6 +210,63 @@ class Task(DefinitionModule.ParameterBindingsHolder, TaskModule.Task):
         if len(unboundParameterIds):
             raise NotImplementedError(
                 'validation for execution of task %s failed >> cannot process unbound parameters %s' % (self.definition().id(), unboundParameterIds))
+        return
+
+
+    def isStoppable(self):
+        """
+        returns whether this task is stoppable.
+        a task is not stoppable if it has completed, killed, or has errored
+        """
+        if self.executeStatus() == Task.EXECUTE_STATUS_COMPLETED:
+            return False
+
+        if self.executeStatus() == Task.EXECUTE_STATUS_ERRORED:
+            return False
+
+        if self.executeStatus() == Task.EXECUTE_STATUS_KILLED:
+            return False
+
+        return True
+
+
+    def isStopped(self):
+        if self.hasParentTask():
+            return self.parentTask().childTaskIsStopped(self)
+        return False
+
+
+    def isPaused(self):
+        if self.hasParentTask():
+            return self.parentTask().childTaskIsPaused(self)
+        raise NotImplementedError
+
+
+    def checkIfShouldContinue(self):
+
+        if self.isStopped():
+            raise ErrorModule.UserStoppedExecution
+        
+        #if self.isPaused():
+        #    raise ErrorModule.UserPausedExecution
+
+        return
+
+
+    def do(self):
+
+        for executeStage in self.__class__.EXECUTE_STEPS:
+            self.checkIfShouldContinue()
+            func = getattr(self, executeStage)
+            result = func()
+            self.executeStepResults()[executeStage] = result
+            self.currentExecuteStepIndex(self.currentExecuteStepIndex()+1)
+            pass
+
+        # if we ever need to return the value of the execution, then
+        # return self.executeStepResults()['executeFunction']
+        return
+
 
     # END class Task
     pass
@@ -167,11 +284,22 @@ class CompositeTask(Task):
         'allChildTasksHaveCompleted',
         'hasGeneratedTasks',
         'isParameterSweepTasksHolder',
-        'executionIsPaused',
         'shouldLimitChildrenConcurrency',
         'childrenConcurrencyLimit'
     ]
     
+    EXECUTE_STEPS = [
+        'notifyParentOnExecutionStart',
+        'pullDataForParameters',
+        'pullDataForBlackboardParameters',
+        'pullParameterBindingsFromDefinition',
+        'validateParameters',
+        'preInitializeChildTasks',
+        'initializeChildTasks',
+        'startNextTasks'
+        ]
+
+
     def __init__(self):
         Task.__init__(self)
         
@@ -268,34 +396,67 @@ class CompositeTask(Task):
         return tasks[0]
     
     
-    def do(self):
+    def pause(self):
+        raise NotImplementedError
 
-        if self.parentTask() is not None:
-            self.parentTask().childTaskIsRunning(self)
 
-        # create tasks for each of the minimal nodes in the graph
-        # add those tasks to the queue
-        # each of the requests being added to the queue
-        # should have a callback 
+    def isStoppable(self):
+        if not Task.isStoppable(self):
+            return False
 
-        self.pullDataForParameters()
-        
-        # see if there are any blackboard parameters to be handled
-        self.pullDataForBlackboardParameters()
-        
-        self.pullParameterBindingsFromDefinition()
-      
-        self.validateParameters()
+        # even if this is running
+        # but if all of its children have completed
+        # then this has effectively completed
+        if self.allChildTasksHaveCompleted():
+            return False
 
-        self.preInitializeChildTasks()
-        
-        self.initializeChildTasks()
-
-        self.startNextTasks()
-        
         return True
 
+
+    def stop(self):
+        # if there are any active children
+        # then kill them
+
+        completedOrErroredFilter = FilterModule.constructOrFilter()
+        completedOrErroredFilter.addFilter(
+            RelationalModule.ColumnValueFilter(
+                'status',
+                FilterModule.EquivalenceFilter(Task.EXECUTE_STATUS_COMPLETED)
+                )
+            )
+        completedOrErroredFilter.addFilter(
+            RelationalModule.ColumnValueFilter(
+                'status',
+                FilterModule.EquivalenceFilter(Task.EXECUTE_STATUS_ERRORED)
+                )
+            )
+
+        stoppableTaskFilter = FilterModule.constructNotFilter()
+        stoppableTaskFilter.addFilter(completedOrErroredFilter)
+
+        for uncompletedTask in self.getChildTasks(stoppableTaskFilter):
+            uncompletedTask.stop()
+
+            # set status to stopped
+            self.setChildTaskStatus(uncompletedTask, Task.EXECUTE_STATUS_KILLED)
+            pass
+
+        self.executeStatus(Task.EXECUTE_STATUS_KILLED)
+
+        return
+
+
+    def resume(self):
+        raise NotImplementedError
+
+
+    def childTaskIsPaused(self, childTask):
+        raise NotImplementedError
+
+    def childTaskIsStopped(self, childTask):
+        return False
     
+
     def initializeChildTasks(self):
         self.taskGenerator().generateReadyTasks(self)
         return
@@ -304,8 +465,10 @@ class CompositeTask(Task):
     def preInitializeChildTasks(self):
         pass
 
-    
+
     def startNextTasks(self):
+
+        # find the definitions that have already been initialized
         theFilter = RelationalModule.ColumnValueFilter(
             'status',
             FilterModule.EquivalenceFilter('initialized')
@@ -347,7 +510,7 @@ class CompositeTask(Task):
         completedPredecessorFilter.addFilter(
             RelationalModule.ColumnValueFilter(
                 'status',
-                FilterModule.EquivalenceFilter('completed')
+                FilterModule.EquivalenceFilter(Task.EXECUTE_STATUS_COMPLETED)
             )
         )
         completedPredecessors = RelationalModule.Table.reduceRetrieve(
@@ -581,24 +744,49 @@ class CompositeTask(Task):
         return row
     
 
-    def childTaskIsRunning(self, task):
+    def getChildTaskStatus(self, task):
+        row = self.getTaskInformation(task)
+        return row.getColumn('status')
+
+    def setChildTaskStatus(self, task, status):
         row = self.getTaskInformation(task)
         
         command = RelationalCommandModule.SetColumnValueCommand(
-            row, 'status', 'running'
+            row, 'status', status
         )
         
         self.automaton().executeCommand(command)
         return
+        
+
+    def postProcessForAllChildTasksHaveCompleted(self):
+
+        # make sure that the values for the parameters
+        # are propogated correctly
+        self.pushDataForBlackboardParameters()
+        self.pushDataForParameters()
+
+        # set the value 
+        self.allChildTasksHaveCompleted(True)
+
+        # here we need to call this explicitly
+        # because for composite tasks, the "do" function
+        # only starts the minimal child tasks
+        # and this is called only after all tasks have completed
+        # so we cannot put this in the callback
+        # like we do for atomic tasks
+        self.notifyParentOfCompletion()
+
+        return
+    
+    
+    def childTaskIsRunning(self, task):
+        self.setChildTaskStatus(task, Task.EXECUTE_STATUS_RUNNING)
+        return
+
 
     def childTaskHasCompleted(self, task):
-        row = self.getTaskInformation(task)
-        
-        command = RelationalCommandModule.SetColumnValueCommand(
-            row, 'status', 'completed'
-        )
-        
-        self.automaton().executeCommand(command)
+        self.setChildTaskStatus(task, Task.EXECUTE_STATUS_COMPLETED)
 
         # TODO: add this as a critical section
 
@@ -616,28 +804,19 @@ class CompositeTask(Task):
         return
 
     
-    def postProcessForAllChildTasksHaveCompleted(self):
-
-        self.pushDataForBlackboardParameters()
-        self.pushDataForParameters()
-        self.allChildTasksHaveCompleted(True)
-        self.notifyParentOfCompletion()
-        return
-    
-    
     def childTaskHasErrored(self, task, errorInfo):
 
-        row = self.getTaskInformation(task)
-        
-        command = RelationalCommandModule.SetColumnValueCommand(
-            row, 'status', 'errored'
-        )
-        
-        self.automaton().executeCommand(command)
+        self.setChildTaskStatus(task, task.executeStatus())
         
         request = self.workRequest()
         request.exception = True
 
+        # we don't actually want to raise an exception
+        # because this is the error callback
+        # instead, we create an exception
+        # and then set that into the request object
+        # so that the receiver of the callback
+        # can use the information appropriately
         error = NotImplementedError('child task %s has errored')
         errorInfo = (type(error), error, errorInfo)
         request.kwds['error info'] = errorInfo
@@ -679,34 +858,68 @@ class DynamicUpdateCompositeTask(CompositeTask):
 
 class AtomicTask(Task):
 
-    ATTRIBUTES = Task.ATTRIBUTES + []
-    
+    ATTRIBUTES = Task.ATTRIBUTES + [
+        ]
+
+    EXECUTE_STEPS = [
+        'notifyParentOnExecutionStart',
+        'configureExecuteEnvironment',
+        'pullDataForParameters',
+        'pullParameterBindingsFromDefinition',
+        'validateParameters',
+        'executeFunction',
+        'pushDataForParameters',
+        # note that we do not need "notifyParentOfCompletion"
+        # because that is part of the completion callback
+        ]
+
+
     def __init__(self):
         Task.__init__(self)
-        
+
         pass
     
     
-    def do(self):
+    def pause(self):
+        self.parentTask().setChildTaskStatus(self, 'paused')
+        raise NotImplementedError
 
-        if self.parentTask() is not None:
-            self.parentTask().childTaskIsRunning(self)
 
-        self.configureExecuteEnvironment()
-        
-        self.pullDataForParameters()
-        
-        self.pullParameterBindingsFromDefinition()
-        
-        self.validateParameters()
-                
+    def resume(self):
+        self.parentTask().setChildTaskStatus(self, EXECUTE_STATUS_RUNNING)
+        raise NotImplementedError
+
+
+    def stop(self):
+
+        # check if the current stage 
+        # is the one executing the function
+        # i.e. executing the process
+        # if so, kill the process
+        if self.currentExecuteStepIndex() == AtomicTask.EXECUTE_STEPS.index('executeFunction'):
+            request = self.workRequest()
+            executeEnv = request.kwds.get('execute environment', None)
+            if executeEnv is not None:
+
+                # the environment should kill task
+                executeEnv.kill(self)
+
+                # reset the execution step index
+                self.currentExecuteStepIndex(-1)
+
+                # set the execute status to being killed
+                self.executeStatus(Task.EXECUTE_STATUS_KILLED)
+
+                pass
+
+        return
+
+
+    def executeFunction(self):
         functionToExecute = self.definition().functionToExecute()
         result = functionToExecute(self)
-
-        self.pushDataForParameters()
-
         return result
-    
+
     
     def configureExecuteEnvironment(self):
         """
@@ -867,6 +1080,7 @@ class TaskGenerator(ResourceModule.Struct):
         ResourceModule.Struct.__init__(self)
         return
     
+
     def pullDataForBlackboardParameters(self, parentTask):
         """
         copies the data from the input parameter of the parent task
@@ -889,8 +1103,8 @@ class TaskGenerator(ResourceModule.Struct):
             targetParameter = parentTask.definition().getParameter(targetParameterId)
             if not targetParameter.portType() == ParameterModule.PORT_TYPE_BLACKBOARD:
                 continue
-            sourceParameterId = parameterConnection.sourceParameter()
 
+            sourceParameterId = parameterConnection.sourceParameter()
 
             parentTask.setParameterBinding(
                 targetParameterId,
@@ -912,36 +1126,30 @@ class TaskGenerator(ResourceModule.Struct):
         if isinstance(definition, DefinitionModule.ReferenceDefinition):
             definition = definition.definitionToReference()
             
-        # create a filter where the source nodes are parentTask,
-        # the target node is parentTask,
-        # and the target parameter is of type blackboard
-        theParameterConnectionFilter = FilterModule.constructAndFilter()
-        theParameterConnectionFilter.addFilter(
-            RelationalModule.ColumnValueFilter(
-                'source node',
-                FilterModule.IdentityFilter(definition)
-            )
-        )
-        theParameterConnectionFilter.addFilter(
-            RelationalModule.ColumnValueFilter(
-                'target node',
-                FilterModule.IdentityFilter(definition.graph())
-            )
-        )
+        theParameterConnectionFilter = \
+            parentTask.getFilterForOwnInternalParameterConnections()
+
 
         for parameterConnection in RelationalModule.Table.reduceRetrieve(
             definition.parameterConnectionsTable(),
             theParameterConnectionFilter,
             ['parameter connection']):
             
-            sourceId = parameterConnection.sourceParameter()
-            sourceParameter = parentTask.definition().getParameter(sourceId)
+            sourceParameterId = parameterConnection.sourceParameter()
+            sourceParameter = parentTask.definition().getParameter(sourceParameterId)
+
             if not sourceParameter.portType() == ParameterModule.PORT_TYPE_BLACKBOARD:
                 continue
-            targetId = parameterConnection.targetParameter()
+
+            if not parentTask.hasParameterBinding(sourceParameterId):
+                raise KeyError("parent task %s only has the following bindings >> %s" % 
+                               (parentTask.definition().name(),
+                                parentTask.parameterBindings()))
+
+            targetParameterId = parameterConnection.targetParameter()
             parentTask.setParameterBinding(
-                targetId,
-                parentTask.getParameterBinding(sourceId)
+                targetParameterId,
+                parentTask.getParameterBinding(sourceParameterId)
             )
             pass
         
@@ -978,7 +1186,6 @@ class TaskGenerator(ResourceModule.Struct):
             x for x in definition.getParametersByFilter(parameterFilter)
         ]
 
-        
         # propagate the parameter bindings into the input parameters
         for referencedParameter in referencedParameters:
 
@@ -1045,8 +1252,6 @@ class TaskGenerator(ResourceModule.Struct):
         parentTaskDefinition = parentTask.definition()
         if isinstance(parentTaskDefinition, DefinitionModule.ReferenceDefinition):
             parentTaskDefinition = parentTaskDefinition.definitionToReference()
-            
-
         # parameter connection filter
         filter = FilterModule.constructAndFilter()
         filter.addFilter(
@@ -1073,8 +1278,9 @@ class TaskGenerator(ResourceModule.Struct):
 
             sourceParameter = sourceNode.getParameter(sourceParameterId)
             sourceParameterType = sourceParameter.portType()
-            if not sourceParameterType == ParameterModule.PORT_TYPE_DATA:
+            if not sourceParameterType == ParameterModule.PORT_TYPE_DATA: 
                 continue
+
             sourceParameterDirection = sourceParameter.portDirection()
             if not (sourceParameterDirection == ParameterModule.PORT_DIRECTION_OUTPUT or 
                     (sourceParameterDirection == ParameterModule.PORT_DIRECTION_INPUT and sourceParameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT))):
@@ -1110,7 +1316,6 @@ class NestTaskGenerator(TaskGenerator):
            not parentTask.hasGeneratedMinimalTasks:
             return True
         
-        # if hasattr(parentTask, 'completedChildTask'):
         if completedChildTask is not None:
             completedDefinition = completedChildTask.definition()
             if len([x for x in completedDefinition.successors()]) is not 0:
@@ -1124,8 +1329,8 @@ class NestTaskGenerator(TaskGenerator):
     
     def generateReadyTasks(self, parentTask, completedChildTask=None):
 
-        if parentTask.executionIsPaused():
-            return
+        #if parentTask.isPaused():
+        #    return
 
         definitionsForNextTasks = []
         if not hasattr(parentTask, 'hasGeneratedMinimalTasks') or \
@@ -1195,7 +1400,7 @@ class NestTaskGenerator(TaskGenerator):
         theFilter.addFilter(
             RelationalModule.ColumnValueFilter(
                 'status',
-                FilterModule.EquivalenceFilter('completed')
+                FilterModule.EquivalenceFilter(Task.EXECUTE_STATUS_COMPLETED)
             )
         )
         completedMaximalDefinitions = RelationalModule.Table.reduceRetrieve(
@@ -1248,7 +1453,6 @@ class LoopTaskGenerator(TaskGenerator):
         
         # increment the state
         # if the previous task just completed
-        # if hasattr(parentTask, 'completedChildTask') and \
         if completedChildTask is not None and \
            not self.hasTransitionedState():
             # here we have to process the state transition
@@ -1285,8 +1489,8 @@ class LoopTaskGenerator(TaskGenerator):
     
     def generateReadyTasks(self, parentTask, completedChildTask=None):
         
-        if parentTask.executionIsPaused():
-            return
+        #if parentTask.isPaused():
+        #    return
 
         # the initial state does not satisfy the continue condition
         if not self.canGenerateMoreTasks(parentTask):
@@ -1336,7 +1540,7 @@ class LoopTaskGenerator(TaskGenerator):
         
         # now ensure that all the tasks have actually completed
         theNotCompletedFilter = FilterModule.constructNotFilter()
-        theNotCompletedFilter.addFilter(FilterModule.EquivalenceFilter('completed'))
+        theNotCompletedFilter.addFilter(FilterModule.EquivalenceFilter(Task.EXECUTE_STATUS_COMPLETED))
         
         taskFilter = RelationalModule.ColumnValueFilter(
             'status',
@@ -1394,8 +1598,8 @@ class BranchTaskGenerator(TaskGenerator):
 
     def generateReadyTasks(self, parentTask, completedChildTask=None):
         
-        if parentTask.executionIsPaused():
-            return
+        #if parentTask.isPaused():
+        #    return
 
         if not self.canGenerateMoreTasks(
             parentTask, completedChildTask=completedChildTask):
@@ -1438,7 +1642,7 @@ class BranchTaskGenerator(TaskGenerator):
         
         # now ensure that all the tasks have actually completed
         theNotCompletedFilter = FilterModule.constructNotFilter()
-        theNotCompletedFilter.addFilter(FilterModule.EquivalenceFilter('completed'))
+        theNotCompletedFilter.addFilter(FilterModule.EquivalenceFilter(Task.EXECUTE_STATUS_COMPLETED))
         
         taskFilter = RelationalModule.ColumnValueFilter(
             'status',
@@ -1513,8 +1717,8 @@ class ParameterSweepTaskGenerator(TaskGenerator):
     
     def generateReadyTasks(self, parentTask, completedChildTask=None):
 
-        if parentTask.executionIsPaused():
-            return
+        #if parentTask.isPaused():
+        #    return
 
         definitionsForNextTasks = []
         
@@ -1683,7 +1887,7 @@ class ParameterSweepTaskGenerator(TaskGenerator):
         
         # now ensure that all the tasks have actually completed
         theNotCompletedFilter = FilterModule.constructNotFilter()
-        theNotCompletedFilter.addFilter(FilterModule.EquivalenceFilter('completed'))
+        theNotCompletedFilter.addFilter(FilterModule.EquivalenceFilter(Task.EXECUTE_STATUS_COMPLETED))
         
         taskFilter = RelationalModule.ColumnValueFilter(
             'status',
